@@ -1,5 +1,6 @@
 # Copyright 2015 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -13,14 +14,11 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import cachetools
 import contextlib
-import threading
 
 from oslo_config import cfg
 from oslo_db import api as db_api
 
-
 _BACKEND_MAPPING = {
     'sqlalchemy': 'mistral.db.v2.sqlalchemy.api',
 }
@@ -29,13 +27,6 @@ IMPL = db_api.DBAPI('sqlalchemy', backend_mapping=_BACKEND_MAPPING)
 
 CONF = cfg.CONF
 
-_ACTION_DEF_CACHE = cachetools.TTLCache(
-    maxsize=1000,
-    ttl=CONF.engine.action_definition_cache_time  # 60 seconds by default
-)
-
-_ACTION_DEF_CACHE_LOCK = threading.RLock()
-
 
 def setup_db():
     IMPL.setup_db()
@@ -179,32 +170,106 @@ def delete_workflow_definitions(**kwargs):
     IMPL.delete_workflow_definitions(**kwargs)
 
 
+# Dynamic actions.
+
+def get_dynamic_action_definition(identifier, namespace='', fields=()):
+    return IMPL.get_dynamic_action_definition(identifier, fields, namespace)
+
+
+def load_dynamic_action_definition(identifier, namespace='', fields=()):
+    return IMPL.load_dynamic_action_definition(identifier, fields, namespace)
+
+
+def create_dynamic_action_definition(values):
+    return IMPL.create_dynamic_action_definition(values)
+
+
+def update_dynamic_action_definition(identifier, values, namespace=''):
+    return IMPL.update_dynamic_action_definition(identifier, values, namespace)
+
+
+def get_dynamic_action_definitions(limit=None, marker=None, sort_keys=None,
+                                   sort_dirs=None, fields=None, **kwargs):
+    return IMPL.get_dynamic_action_definitions(
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        fields=fields,
+        **kwargs
+    )
+
+
+def delete_dynamic_action_definition(identifier, namespace=''):
+    return IMPL.delete_dynamic_action_definition(identifier, namespace)
+
+
+def delete_dynamic_action_definitions(**kwargs):
+    return IMPL.delete_dynamic_action_definitions(**kwargs)
+
+
+# Code sources.
+
+def get_code_source(identifier, namespace='', fields=()):
+    return IMPL.get_code_source(identifier, fields, namespace=namespace)
+
+
+def load_code_source(identifier, namespace='', fields=()):
+    return IMPL.load_code_source(identifier, fields, namespace=namespace)
+
+
+def create_code_source(values):
+    return IMPL.create_code_source(values)
+
+
+def update_code_source(identifier, values, namespace=''):
+    return IMPL.update_code_source(identifier, values, namespace=namespace)
+
+
+def get_code_sources(limit=None, marker=None, sort_keys=None,
+                     sort_dirs=None, fields=None, **kwargs):
+    return IMPL.get_code_sources(
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        fields=fields,
+        **kwargs
+    )
+
+
+def delete_code_source(identifier, namespace=''):
+    return IMPL.delete_code_source(identifier, namespace=namespace)
+
+
+def delete_code_sources(**kwargs):
+    return IMPL.delete_code_sources(**kwargs)
+
+
 # Action definitions.
 
 def get_action_definition_by_id(id, fields=()):
     return IMPL.get_action_definition_by_id(id, fields=fields)
 
 
-def get_action_definition(name, fields=()):
-    return IMPL.get_action_definition(name, fields=fields)
+def get_action_definition(name, fields=(), namespace=''):
+    return IMPL.get_action_definition(name, fields=fields, namespace=namespace)
 
 
-def load_action_definition(name, fields=()):
+def load_action_definition(name, fields=(), namespace=''):
     """Unlike get_action_definition this method is allowed to return None."""
-    with _ACTION_DEF_CACHE_LOCK:
-        action_def = _ACTION_DEF_CACHE.get(name)
+    action_def = IMPL.load_action_definition(
+        name,
+        fields=fields,
+        namespace=namespace
+    )
 
+    # If action definition was not found in the workflow namespace,
+    # check in the default namespace
     if action_def:
         return action_def
 
-    action_def = IMPL.load_action_definition(name, fields=fields)
-
-    with _ACTION_DEF_CACHE_LOCK:
-        _ACTION_DEF_CACHE[name] = (
-            action_def.get_clone() if action_def else None
-        )
-
-    return action_def
+    return IMPL.load_action_definition(name, fields=fields, namespace='')
 
 
 def get_action_definitions(limit=None, marker=None, sort_keys=None,
@@ -230,8 +295,8 @@ def create_or_update_action_definition(name, values):
     return IMPL.create_or_update_action_definition(name, values)
 
 
-def delete_action_definition(name):
-    return IMPL.delete_action_definition(name)
+def delete_action_definition(name, namespace=''):
+    return IMPL.delete_action_definition(name, namespace=namespace)
 
 
 def delete_action_definitions(**kwargs):
@@ -459,17 +524,17 @@ def get_scheduled_jobs_count(**kwargs):
 
 # Cron triggers.
 
-def get_cron_trigger(identifier):
-    return IMPL.get_cron_trigger(identifier)
+def get_cron_trigger(identifier, fields=()):
+    return IMPL.get_cron_trigger(identifier, fields=fields)
 
 
-def get_cron_trigger_by_id(id):
-    return IMPL.get_cron_trigger_by_id(id)
+def get_cron_trigger_by_id(id, fields=()):
+    return IMPL.get_cron_trigger_by_id(id, fields=fields)
 
 
-def load_cron_trigger(identifier):
+def load_cron_trigger(identifier, fields=()):
     """Unlike get_cron_trigger this method is allowed to return None."""
-    return IMPL.load_cron_trigger(identifier)
+    return IMPL.load_cron_trigger(identifier, fields=fields)
 
 
 def get_cron_triggers(**kwargs):
@@ -528,18 +593,17 @@ def delete_cron_triggers(**kwargs):
 
 # Environments.
 
-def get_environment(name):
-    return IMPL.get_environment(name)
+def get_environment(name, fields=()):
+    return IMPL.get_environment(name, fields=fields)
 
 
-def load_environment(name):
+def load_environment(name, fields=()):
     """Unlike get_environment this method is allowed to return None."""
-    return IMPL.load_environment(name)
+    return IMPL.load_environment(name, fields=fields)
 
 
 def get_environments(limit=None, marker=None, sort_keys=None,
                      sort_dirs=None, **kwargs):
-
     return IMPL.get_environments(
         limit=limit,
         marker=marker,
@@ -603,12 +667,12 @@ def delete_resource_members(**kwargs):
 
 # Event triggers.
 
-def get_event_trigger(id, insecure=False):
-    return IMPL.get_event_trigger(id, insecure)
+def get_event_trigger(id, insecure=False, fields=()):
+    return IMPL.get_event_trigger(id, insecure, fields=fields)
 
 
-def load_event_trigger(id, insecure=False):
-    return IMPL.load_event_trigger(id, insecure)
+def load_event_trigger(id, insecure=False, fields=()):
+    return IMPL.load_event_trigger(id, insecure, fields=fields)
 
 
 def get_event_triggers(insecure=False, limit=None, marker=None, sort_keys=None,
