@@ -2,6 +2,7 @@
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -19,6 +20,8 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
 
+from mistral_lib import actions as ml_actions
+
 from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models as db_models
@@ -27,8 +30,8 @@ from mistral.engine import base
 from mistral.engine import post_tx_queue
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions
-from mistral import utils as u
 from mistral.workflow import states
+from mistral_lib import utils as u
 
 
 # Submodules of mistral.engine will throw NoSuchOptError if configuration
@@ -43,7 +46,7 @@ class DefaultEngine(base.Engine):
     @post_tx_queue.run
     @profiler.trace('engine-start-workflow', hide_args=True)
     def start_workflow(self, wf_identifier, wf_namespace='', wf_ex_id=None,
-                       wf_input=None, description='', **params):
+                       wf_input=None, description='', async_=False, **params):
         if wf_namespace:
             params['namespace'] = wf_namespace
 
@@ -81,29 +84,38 @@ class DefaultEngine(base.Engine):
     @db_utils.retry_on_db_error
     @post_tx_queue.run
     def start_action(self, action_name, action_input,
-                     description=None, **params):
+                     description=None, namespace='', **params):
         with db_api.transaction():
-            action = action_handler.build_action_by_name(action_name)
+            engine_action = action_handler.build_action_by_name(
+                action_name,
+                namespace=namespace
+            )
 
-            action.validate_input(action_input)
+            action_desc = engine_action.action_desc
+
+            action_desc.check_parameters(action_input)
 
             sync = params.get('run_sync')
             save = params.get('save_result')
             target = params.get('target')
             timeout = params.get('timeout')
 
-            is_action_sync = action.is_sync(action_input)
+            # In order to know if it's sync or not we have to instantiate
+            # the actual runnable action.
+            action = action_desc.instantiate(action_input, {})
+
+            is_action_sync = action.is_sync()
 
             if sync and not is_action_sync:
                 raise exceptions.InputException(
                     "Action does not support synchronous execution.")
 
             if not sync and (save or not is_action_sync):
-                action.schedule(action_input, target, timeout=timeout)
+                engine_action.schedule(action_input, target, timeout=timeout)
 
-                return action.action_ex.get_clone()
+                return engine_action.action_ex.get_clone()
 
-            output = action.run(
+            output = engine_action.run(
                 action_input,
                 target,
                 save=False,
@@ -120,7 +132,8 @@ class DefaultEngine(base.Engine):
                     description=description,
                     input=action_input,
                     output=output.to_dict(),
-                    state=state
+                    state=state,
+                    workflow_namespace=namespace
                 )
 
             action_ex_id = u.generate_unicode_uuid()
@@ -132,7 +145,8 @@ class DefaultEngine(base.Engine):
                 'input': action_input,
                 'output': output.to_dict(),
                 'state': state,
-                'is_sync': is_action_sync
+                'is_sync': is_action_sync,
+                'workflow_namespace': namespace
             }
 
             return db_api.create_action_execution(values)
@@ -145,6 +159,11 @@ class DefaultEngine(base.Engine):
         with db_api.transaction():
             if wf_action:
                 action_ex = db_api.get_workflow_execution(action_ex_id)
+                # If result is None it means that it's a normal subworkflow
+                # output and we just need to fetch it from the model.
+                # This is just an optimization to not send data over RPC
+                if result is None:
+                    result = ml_actions.Result(data=action_ex.output)
             else:
                 action_ex = db_api.get_action_execution(action_ex_id)
 
@@ -162,7 +181,6 @@ class DefaultEngine(base.Engine):
                 action_ex = db_api.get_workflow_execution(action_ex_id)
             else:
                 action_ex = db_api.get_action_execution(action_ex_id)
-
             action_handler.on_action_update(action_ex, state)
 
             return action_ex.get_clone()
@@ -179,13 +197,19 @@ class DefaultEngine(base.Engine):
 
     @db_utils.retry_on_db_error
     @post_tx_queue.run
-    def rerun_workflow(self, task_ex_id, reset=True, env=None):
+    def rerun_workflow(self, task_ex_id, reset=True, skip=False, env=None):
         with db_api.transaction():
             task_ex = db_api.get_task_execution(task_ex_id)
 
             wf_ex = task_ex.workflow_execution
 
-            wf_handler.rerun_workflow(wf_ex, task_ex, reset=reset, env=env)
+            wf_handler.rerun_workflow(
+                wf_ex,
+                task_ex,
+                reset=reset,
+                skip=skip,
+                env=env
+            )
 
             return wf_ex.get_clone()
 
@@ -215,20 +239,17 @@ class DefaultEngine(base.Engine):
 
     @db_utils.retry_on_db_error
     @post_tx_queue.run
-    def report_running_actions(self, action_ex_ids):
+    def process_action_heartbeats(self, action_ex_ids):
         with db_api.transaction():
-            now = u.utc_now_sec()
-
             for exec_id in action_ex_ids:
                 try:
-                    db_api.update_action_execution(
-                        exec_id,
-                        {"last_heartbeat": now},
-                        insecure=True
-                    )
+                    db_api.update_action_execution_heartbeat(exec_id)
                 except exceptions.DBEntityNotFoundError:
-                    LOG.debug("Action execution heartbeat update failed. {}"
-                              .format(exec_id), exc_info=True)
+                    LOG.debug(
+                        "Action execution heartbeat update failed. {}"
+                        .format(exec_id),
+                        exc_info=True
+                    )
                     # Ignore this error and continue with the
                     # remaining ids.
                     pass
