@@ -32,6 +32,11 @@ from mistral.workflow import base as wf_base
 from mistral.workflow import commands as wf_cmds
 from mistral.workflow import states
 
+# TODO(rakhmerov): At some point we need to completely switch to
+# exceptions from mistral_lib. For the time being though we'll
+# have to catch MistralException from both 'mistral' and 'mistral-lib'.
+from mistral_lib import exceptions as mistral_lib_exc
+
 
 """Responsible for running tasks and handling results."""
 
@@ -66,7 +71,7 @@ def run_task(wf_cmd):
             _schedule_refresh_task_state(task.task_ex.id)
 
         task.run()
-    except exc.MistralException as e:
+    except (exc.MistralException, mistral_lib_exc.MistralException) as e:
         wf_ex = wf_cmd.wf_ex
         task_spec = wf_cmd.task_spec
 
@@ -82,15 +87,23 @@ def run_task(wf_cmd):
     _check_affected_tasks(task)
 
 
-def mark_task_running(task_ex, wf_spec):
-    task = _build_task_from_execution(wf_spec, task_ex)
+@profiler.trace('task-handler-skip-task', hide_args=True)
+def skip_task(wf_cmd):
+    """Skip workflow task.
 
-    old_task_state = task_ex.state
+    :param wf_cmd: Workflow command.
+    """
+    task = _build_task_from_command(wf_cmd)
+    task.complete(states.SKIPPED, "Task was skipped.", skip=True)
+    _check_affected_tasks(task)
+    return
+
+
+def mark_task_running(task_ex, wf_spec):
+    task = build_task_from_execution(wf_spec, task_ex)
 
     task.set_state(states.RUNNING, None, False)
 
-    task.notify(old_task_state, states.RUNNING)
-
 
 @profiler.trace('task-handler-on-action-complete', hide_args=True)
 def _on_action_complete(action_ex):
@@ -118,7 +131,7 @@ def _on_action_complete(action_ex):
 
     try:
         task.on_action_complete(action_ex)
-    except exc.MistralException as e:
+    except (exc.MistralException, mistral_lib_exc.MistralException) as e:
         wf_ex = task_ex.workflow_execution
 
         msg = ("Failed to handle action completion [error=%s, wf=%s, task=%s,"
@@ -173,7 +186,7 @@ def _on_action_update(action_ex):
             # then resume the parent workflow execution.
             wf_handler.resume_workflow(wf_ex)
 
-    except exc.MistralException as e:
+    except (exc.MistralException, mistral_lib_exc.MistralException) as e:
         wf_ex = task_ex.workflow_execution
 
         msg = ("Failed to handle action update [error=%s, wf=%s, task=%s,"
@@ -209,13 +222,9 @@ def force_fail_task(task_ex, msg, task=None):
             task_ex.workflow_execution_id
         )
 
-        task = _build_task_from_execution(wf_spec, task_ex)
+        task = build_task_from_execution(wf_spec, task_ex)
 
-    old_task_state = task_ex.state
     task.set_state(states.ERROR, msg)
-    task.notify(old_task_state, states.ERROR)
-
-    task.save_finished_time()
 
     wf_handler.force_fail_workflow(task_ex.workflow_execution, msg)
 
@@ -228,7 +237,7 @@ def continue_task(task_ex):
         task_ex.workflow_execution_id
     )
 
-    task = _build_task_from_execution(wf_spec, task_ex)
+    task = build_task_from_execution(wf_spec, task_ex)
 
     try:
         task.set_state(states.RUNNING, None)
@@ -257,11 +266,11 @@ def complete_task(task_ex, state, state_info):
         task_ex.workflow_execution_id
     )
 
-    task = _build_task_from_execution(wf_spec, task_ex)
+    task = build_task_from_execution(wf_spec, task_ex)
 
     try:
         task.complete(state, state_info)
-    except exc.MistralException as e:
+    except (exc.MistralException, mistral_lib_exc.MistralException) as e:
         wf_ex = task_ex.workflow_execution
 
         msg = (
@@ -278,6 +287,9 @@ def complete_task(task_ex, state, state_info):
 
 @profiler.trace('task-handler-check-affected-tasks', hide_args=True)
 def _check_affected_tasks(task):
+    # TODO(rakhmerov): this method should eventually move into
+    # the class Task. The obvious signal is the only argument
+    # that it takes.
     if not task.is_completed():
         return
 
@@ -324,7 +336,7 @@ def _check_affected_tasks(task):
         )
 
 
-def _build_task_from_execution(wf_spec, task_ex):
+def build_task_from_execution(wf_spec, task_ex):
     return _create_task(
         task_ex.workflow_execution,
         wf_spec,
@@ -367,6 +379,19 @@ def _build_task_from_command(cmd):
 
         return task
 
+    if isinstance(cmd, wf_cmds.SkipTask):
+        task = _create_task(
+            cmd.wf_ex,
+            cmd.wf_spec,
+            spec_parser.get_task_spec(cmd.task_ex.spec),
+            cmd.ctx,
+            task_ex=cmd.task_ex,
+            unique_key=cmd.task_ex.unique_key,
+            triggered_by=cmd.triggered_by,
+        )
+
+        return task
+
     raise exc.MistralError('Unsupported workflow command: %s' % cmd)
 
 
