@@ -130,7 +130,7 @@ class DirectWorkflowController(base.WorkflowController):
             if not (t_s or t_n in commands.ENGINE_CMD_CLS):
                 raise exc.WorkflowException("Task '%s' not found." % t_n)
             elif not t_s:
-                t_s = self.wf_spec.get_tasks()[task_ex.name]
+                t_s = self.wf_spec.get_task(task_ex.name)
 
             triggered_by = [
                 {
@@ -159,7 +159,7 @@ class DirectWorkflowController(base.WorkflowController):
         return cmds
 
     def _configure_if_join(self, cmd):
-        if not isinstance(cmd, commands.RunTask):
+        if not isinstance(cmd, (commands.RunTask, commands.RunExistingTask)):
             return
 
         if not cmd.task_spec.get_join():
@@ -171,6 +171,17 @@ class DirectWorkflowController(base.WorkflowController):
     def _get_join_unique_key(self, cmd):
         return 'join-task-%s-%s' % (self.wf_ex.id, cmd.task_spec.get_name())
 
+    def rerun_tasks(self, task_execs, reset=True):
+        cmds = super(DirectWorkflowController, self).rerun_tasks(
+            task_execs,
+            reset
+        )
+
+        for cmd in cmds:
+            self._configure_if_join(cmd)
+
+        return cmds
+
     # TODO(rakhmerov): Need to refactor this method to be able to pass tasks
     # whose contexts need to be merged.
     def evaluate_workflow_final_context(self):
@@ -264,6 +275,7 @@ class DirectWorkflowController(base.WorkflowController):
 
         return res and not task_ex.has_next_tasks
 
+    @profiler.trace('direct-wf-controller-find-next-tasks', hide_args=True)
     def _find_next_tasks(self, task_ex, ctx):
         t_n = task_ex.name
         t_s = task_ex.state
@@ -283,18 +295,34 @@ class DirectWorkflowController(base.WorkflowController):
             for name, cond, params in self.wf_spec.get_on_error_clause(t_n):
                 if not cond or expr.evaluate(cond, ctx_view):
                     params = expr.evaluate_recursively(params, ctx_view)
+
                     result.append((name, params, 'on-error'))
 
-        if t_s == states.SUCCESS:
+        skip_is_empty = False
+        if t_s == states.SKIPPED:
+            for name, cond, params in self.wf_spec.get_on_skip_clause(t_n):
+                if not cond or expr.evaluate(cond, ctx_view):
+                    params = expr.evaluate_recursively(params, ctx_view)
+                    result.append((name, params, 'on-skip'))
+
+            # We should go to 'on-success' branch in case of
+            # skipping task with no 'on-skip' specified.
+            if len(result) == 0:
+                skip_is_empty = True
+
+        if t_s == states.SUCCESS or skip_is_empty:
             for name, cond, params in self.wf_spec.get_on_success_clause(t_n):
                 if not cond or expr.evaluate(cond, ctx_view):
                     params = expr.evaluate_recursively(params, ctx_view)
+
                     result.append((name, params, 'on-success'))
 
-        if states.is_completed(t_s) and not states.is_cancelled(t_s):
+        if states.is_completed(t_s) \
+                and not states.is_cancelled_or_skipped(t_s):
             for name, cond, params in self.wf_spec.get_on_complete_clause(t_n):
                 if not cond or expr.evaluate(cond, ctx_view):
                     params = expr.evaluate_recursively(params, ctx_view)
+
                     result.append((name, params, 'on-complete'))
 
         return result
